#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include "Maths.h"
#include "math/Vec2f.h"

#define SCREEN_WIDTH    640
#define SCREEN_HEIGHT   480

int maxLevel = 0;//5;

void vonKoch(Vec2f p1, Vec2f p2, int level)
{
    if (level >= maxLevel)
    {
        gfx.line(p1.x, p1.y, p2.x, p2.y, Color(255, 255, 255));
        return;
    }

    Vec2f   n[7];

    Vec2f   v  = (p2 - p1) / 4;
    Vec2f   v2 = v;
    v2.rotate(DEG_TO_RAD(-90));

    n[0] = p1 + v;
    n[6] = p2 - v;

    n[1] = n[0] + v2;
    n[5] = n[6] - v2;

    n[2] = n[1] + v;
    n[4] = n[5] - v;

    n[3] = (n[2] + n[4]) / 2;

    vonKoch(  p1, n[0], level + 1);
    vonKoch(n[0], n[1], level + 1);
    vonKoch(n[1], n[2], level + 1);
    vonKoch(n[2], n[3], level + 1);
    vonKoch(n[3], n[4], level + 1);
    vonKoch(n[4], n[5], level + 1);
    vonKoch(n[5], n[6], level + 1);
    vonKoch(n[6],   p2, level + 1);
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Von Koch 8", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    Vec2f   left (0             , SCREEN_HEIGHT/2);
    Vec2f   right(SCREEN_WIDTH-1, SCREEN_HEIGHT/2);

    int cnt = 0;

    // wait until we quit
    while (sys.isQuitRequested() == false)
    {
        if (cnt == 0)
        {
            gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));
            vonKoch(left, right, 0);
        }

        cnt++;

        if (maxLevel < 4 && cnt == 50)
        {
            cnt = 0;
            maxLevel++;
        }

        gfx.render();
        sys.wait(20);
        sys.processEvents();
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
